=	export data in intel's hex format
=	exporter script for mirkes.de's tiny hex editor
=
=	(C) markus stephany, tinyhexer[at]mirkes[dot]de
=
=  params: 1=out file, 2=start, 3=count

INCLUDE 'def.mps'
INCLUDE 'gui.mps'

option GLOBALVARS, 1
option target, export


var count dword start dword outfn text outf file inf file 
var lastaddr dword:= last address to be exported
var addrtype byte:= type of the address record (0 = none, 1 = segment, 2 = linear)
var written dword:= number of bytes written
var curseg word:= current segment/linear offset
curseg = 65535
var skipbyte byte:= skip byte for unoccupied space
var doskip byte:= is skip byte set?
var lineaddr dword

var voffset dword:= virtual offset, depending on the range of the input file (and the range of the output file, if 
                  = importing to an existing file             

INCLUDE '*.lng'

= get the export file name
if param_count > 0
  outfn = params(0)
else
  outfn = asksavefilename(__STR1__, __STR2__, 'hex')
endif

inf = fileopen('::current')
start = 0
count = filesize(inf)

= get amount of data to store
if param_count > 1
  start = dword(params(1))
  fileseek inf start
  if param_count > 2
    count = dword(params(2))
  endif
endif

voffset = filegetprop(inf, '!Offset')+start
lastaddr = start+count-1
skipbyte = filegetprop(inf, 'FillByte')


var lb text


@@start
= get file options thru a dialog
if GuiExists('dlgExportIntelHex'):GuiDestroy('dlgExportIntelHex'):endif

call create_form


var b_ok byte
= show dialog
b_ok = GuiShowModal('dlgExportIntelHex') == IDOK
GuiDestroy('dlgExportIntelHex'):= destroy dialog
if b_ok

  = open out file
  outf = fileopen(outfn,'c')
  

  = write all data
  var cur dword data text cur_v dword
  cur = start
  cur_v = voffset 
  fileseek inf, start
  call(lb):= use function as it can jump to variable targets (needed to distinguish between segment/linear/no address records)
  
  = write eof record
  filewrite outf ":00000001FF\r\n"  

  fileclose inf
  showprogress 100

  fileclose outf

endif
end

var c_read char max_v dword


@@rw_norm:= read/write data
repeat
  showprogress count written
  if doskip:call findstart:endif
  lineaddr = cur_v
  = calculate max_v 
  max_v = cur_v +31:= max 32 bytes per record
  data = ''
  repeat
    if cur > lastaddr:goto w_norm:endif
    if (fileread(inf, @c_read))
      inc cur
      inc cur_v
      if doskip and (c_read == skipbyte):goto w_norm:endif
      concat data, c_read
    else
      goto w_norm
    endif  
  until cur_v > max_v

@@w_norm
  if data != ''
    inc written, textlen(data)
    call writerec data2textbe(byte(textlen(data)))+data2textbe(word(lineaddr))+data2textbe(byte(0))+data
  endif 
until cur > lastaddr
return

@@rw_seg:= read/write data and watch for segment offset changes
repeat
  showprogress count written
  if doskip:call findstart:endif
  lineaddr = cur_v
  = calculate max_v 
  max_v = cur_v +31:= max 32 bytes per record
  data = ''
  repeat
    if cur > lastaddr:goto w_seg:endif
    if (cur_V >> 16) != (lineaddr >> 16):goto w_seg:endif:= segment addresses wrap at 64k boundary, so break before segment change
    if (fileread(inf, @c_read))
      inc cur
      inc cur_v
      if doskip and (c_read == skipbyte):goto w_seg:endif
      concat data, c_read
    else
      goto w_seg
    endif  
  until cur_v > max_v

@@w_seg
  if data != ''
    if (lineaddr >> 16) != curseg
      curseg = lineaddr >> 16
      call writerec data2textbe(byte(2))+data2textbe(word(0))+data2textbe(byte(2))+data2textbe(word(curseg << 12))
    endif
    inc written, textlen(data)
    call writerec data2textbe(byte(textlen(data)))+data2textbe(word(lineaddr))+data2textbe(byte(0))+data
  endif 
until cur > lastaddr
return

@@rw_linear:= read/write data and watch for linear offset changes
repeat
  showprogress count written
  if doskip:call findstart:endif
  lineaddr = cur_v
  = calculate max_v 
  max_v = cur_v +31:= max 32 bytes per record
  data = ''
  repeat
    if cur > lastaddr:goto w_linear:endif
    if (fileread(inf, @c_read))
      inc cur
      inc cur_v
      if doskip and (c_read == skipbyte):goto w_linear:endif
      concat data, c_read
    else
      goto w_linear
    endif  
  until cur_v > max_v

@@w_linear
  if data != ''
    if (lineaddr >> 16) != curseg
      curseg = lineaddr >> 16
      call writerec data2textbe(byte(2))+data2textbe(word(0))+data2textbe(byte(4))+data2textbe(word(curseg))
    endif
    inc written, textlen(data)
    call writerec data2textbe(byte(textlen(data)))+data2textbe(word(lineaddr))+data2textbe(byte(0))+data
  endif 
until cur > lastaddr
return

@@findstart:= read over skipbyte
if cur > lastaddr:return:endif
fileread inf c_read
inc cur
inc cur_v
if (c_read != skipbyte) or (cur > lastaddr)
  inc cur -1
  inc cur_v -1
  fileseek inf, -1, file_current
  return
endif
goto findstart

@@writerec:= build checksum and write the record
var w_dat text w_sum byte w_index word w_len word
pop w_dat
w_sum = 0
w_index = 1
w_len = textlen(w_dat)+1
while w_index < w_len
  inc w_sum, byte(char(textcopy(w_dat,w_index,1)))
  inc w_index
endwhile
filewrite outf ":"+bin2hex(w_dat)+bin2hex(data2textbe(byte(256-w_sum)))+"\r\n"
return

@@check_dialog:= check dialog values
var cdt text c_offs dword

GuiSetProp('dlgExportIntelHex.edOffs', 'Focused', True)
cdt = GuiGetProp('dlgExportIntelHex.edoffs', 'Text')
if (not IsNumber(cdt)):goto check_dialog_num_error:endif
c_offs = dword(cdt)

GuiSetProp('dlgExportIntelHex.edSkip', 'Focused', True)
cdt = GuiGetProp('dlgExportIntelHex.edSkip', 'Text')
if (not IsNumber(cdt)):goto check_dialog_num_error:endif
skipbyte = byte(cdt)

voffset = c_offs
doskip = GuiGetProp('dlgExportIntelHex.cbSkip', 'Checked')

addrtype = guigetprop('dlgExportIntelHex.rgAddr', 'ItemIndex')

GuiSetProp('dlgExportIntelHex.rgAddr', 'Focused', True)
= prepare read and write procs depending on addrtype
if addrtype == 1
  lb = "rw_seg"
  if (lastaddr + voffset) > 0xfffff
    = address range to large for 20 bit records
    if MsgBox(__STR3__, MB_ICONHAND or MB_YESNO) == IDNO
      goto check_dialog_cancel_close
    endif
  endif
else
  if addrtype == 2
    lb = "rw_linear"
  else
    lb = "rw_norm"
  endif
  if addrtype == 0 
    if (lastaddr + voffset) > 0xffff
      = address range to large for 16 bit records
      if MsgBox(__STR4__, MB_ICONHAND or MB_YESNO) == IDNO
        goto check_dialog_cancel_close
      endif
    endif
  endif
endif


return

@@check_dialog_num_error
msgbox __STR5__, MB_ICONSTOP
@@check_dialog_cancel_close
GuiSetProp('dlgExportIntelHex', 'ModalResult', 0)
return



